/* Ce programme a pour but de faire fonctionner la carte arduino uno de la carte de mesure eolecole.
 *  Il affiche à la fois des informations sur l'écran LCD de la carte, ou sur le moniteur série d'un 
 *  ordinateur
 * Vous pouvez l'adapter à vos besoin.
 * 
 * Doc de l'ecran lcd https://lastminuteengineers.com/arduino-1602-character-lcd-tutorial/
 * 
 * Créé par : Simon et Martin - www.eolecole.fr - contact@eolecole.fr
 */
#include <LiquidCrystal.h> //bibliothèque de l'écran lcd

//Paramètrage des pins
#define RPM_PIN  3 
#define UPROD_PIN A0
#define UMESURE_PIN A1
#define UINTENSITE_PIN A2 //Ui
#define LED_ON 10
#define LED_ERR 11
#define SWITCH_PIN 2

LiquidCrystal lcd(8, 9, 4, 5, 6, 7); // Paramètrage de l'écran LCD : (rs, enable, d4, d5, d6, d7)

//Initialisation des variables
double rpm = 0;
unsigned long pulse = 0; 
unsigned long t0 = 0, t1 = 0; 
double courant = 0, tensionProd=0, tensionMes=0;

void setup() {
  Serial.begin(9600);
  lcd.begin(16, 2); //Nombre de colonnes et de ligne de l'écran
  lcd.clear();
  lcd.print("Eolecole-mesure");

  //Réglage des entrées et sorties
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(LED_ON, OUTPUT);
  pinMode(LED_ERR, OUTPUT);
  pinMode(RPM_PIN, INPUT);
  pinMode(UPROD_PIN, INPUT);
  pinMode(UMESURE_PIN, INPUT);
  pinMode(COURANT_PIN, INPUT);
  pinMode(SWITCH_PIN, INPUT);
  digitalWrite(LED_ERR, HIGH);

  attachInterrupt(digitalPinToInterrupt(RPM_PIN), RPM, RISING); //Rising permet de compter les passages de 0 à 1 du signal de l'AOP rpm, donc la durée d'une periode de phase 
  attachInterrupt(digitalPinToInterrupt(SWITCH_PIN), AFFICHEMESURECOURANT, HIGH); //Pour changer de mode quand le bouton est appuyé

  Serial.println("----------- Eolecole - Mesures -----------"); 
}


void loop() {
  //Led de fonctionnement
  digitalWrite(LED_BUILTIN, HIGH);
  digitalWrite(LED_ON, LOW);
  delay(80);
  digitalWrite(LED_BUILTIN, LOW);
  digitalWrite(LED_ON, HIGH);
  delay(920);   

  //MESURE DE LA VITESSE DE ROTATION (RPM) de l'éolienne quand elle tourne (rpm est non nul)
  if(rpm){
    Serial.print(rpm,0); // le 0 enlève les chiffres apprès la virgule
    Serial.print(" rpm ");
    Serial.print(pulse);
    Serial.println(" micro s");
    
    lcd.clear();
    lcd.print(rpm,0);
    lcd.print(" rpm");
    lcd.setCursor(0, 1); 
    lcd.print(pulse);
    lcd.print(" micro s ");
    
  }else {
    Serial.println("Allez tourne !");
    lcd.clear();
    lcd.print("Allez tourne !");
  }
  rpm = 0;
  pulse = 0;

  //MESURE COURANT
  courant = analogRead(UINTENSITE_PIN); //Lecture de Ui
  courant = courant*5/1023;
  Serial.println();
  Serial.print("Courant (mA): "); 
  Serial.println(courant); //Rmes (10 Ohm) x Amplification (100) =  1000 ; donc courant (mA) = (Ui/(10*100))*1000 = Ui

  //Tension produite par l'éolienne
  tensionProd=analogRead(UPROD_PIN);
  tensionProd=tensionProd*2*5/1023; // x2 car pont diviseur
  Serial.print("Tension produite : ");
  Serial.println(tensionProd);
  
  //Tension de mesure du courant (trop imprécise pour etre exploitée sans amplification)
  tensionMes=analogRead(UMESURE_PIN);
  tensionMes=tensionMes*5/1023;
  Serial.print("Tension de mesure courant (trop imprécise) : ");
  Serial.println(tensionMes);
  Serial.println();

}

void RPM() {
  t0 = t1;
  t1 = micros();

  pulse = (t1 - t0);

  rpm = 30000000 / (t1 - t0);  // micro / 1000000 -> secondes / 60 -> minutes (tour par minute) / 2 (4 aimants sur le rotor) = 30000000
}

void AFFICHEMESURECOURANT(){
  lcd.clear();
  lcd.print("I = ");
  lcd.print(courant);
  lcd.print("mA");
  lcd.setCursor(0, 1);
  lcd.print("Tension : ");
  lcd.print(tensionProd);
  lcd.print(" V");
  delay(200000); //Delay important pour une raison inconnu. Les interruptions semble s'interrompre entre elles
}
